// Copyright (c) 2022 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include <winsock2.h>

#include "WSAInterface.h"

//------------------------------------------------------------------------------

using hu::varadiistvan::scpl::io::WSAInterface;

//------------------------------------------------------------------------------

WSAInterface::WSAInterface()
{
    WSADATA wsaData;
    WSAStartup(MAKEWORD(2, 2), &wsaData);
}

//------------------------------------------------------------------------------

WSAInterface::~WSAInterface()
{
    WSACleanup();
}

//------------------------------------------------------------------------------

HANDLE WSAInterface::getInvalidSocket()
{
    return reinterpret_cast<HANDLE>(INVALID_SOCKET);
}

//------------------------------------------------------------------------------

bool WSAInterface::isInvalidSocket(HANDLE handle)
{
    return handle==reinterpret_cast<HANDLE>(INVALID_SOCKET);
}

//------------------------------------------------------------------------------

HANDLE WSAInterface::socket(int af, int type, int protocol)
{
    return reinterpret_cast<HANDLE>(::socket(af, type, protocol));
}

//------------------------------------------------------------------------------

int WSAInterface::eventSelect(HANDLE socket, HANDLE event, long eventMask)
{
    return WSAEventSelect(reinterpret_cast<SOCKET>(socket),
                          event, eventMask);
}

//------------------------------------------------------------------------------

int WSAInterface::setsockopt(HANDLE socket, int level, int optname,
                             const char* optval, int optlen)
{
    return ::setsockopt(reinterpret_cast<SOCKET>(socket), level, optname,
                        optval, optlen);
}

//------------------------------------------------------------------------------

int WSAInterface::bind(HANDLE socket, const struct sockaddr* name, int namelen)
{
    return ::bind(reinterpret_cast<SOCKET>(socket), name, namelen);
}

//------------------------------------------------------------------------------

int WSAInterface::listen(HANDLE socket, int backlog)
{
    return ::listen(reinterpret_cast<SOCKET>(socket), backlog);
}

//------------------------------------------------------------------------------

HANDLE WSAInterface::accept(HANDLE socket, struct sockaddr* addr, int* addrlen)
{
    return reinterpret_cast<HANDLE>(::accept(reinterpret_cast<SOCKET>(socket),
                                             addr, addrlen));
}

//------------------------------------------------------------------------------

int WSAInterface::connect(HANDLE socket, const struct sockaddr* name, int namelen)
{
    return ::connect(reinterpret_cast<SOCKET>(socket), name, namelen);
}

//------------------------------------------------------------------------------

int WSAInterface::closesocket(HANDLE socket)
{
    return ::closesocket(reinterpret_cast<SOCKET>(socket));
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
